@@ -1,3 +1,4 @@
+# $Id: EMBL.pm,v 1.17 2001/01/28 06:45:08 lapp Exp $
 #
 # BioPerl module for Bio::Index::Abstract
 #
@@ -55,9 +56,10 @@ User feedback is an integral part of the evolution of this and other
 Bioperl modules. Send your comments and suggestions preferably to one
 of the Bioperl mailing lists.  Your participation is much appreciated.
 
-   bioperl-l@bioperl.org             - General discussion
-   bioperl-guts-l@bioperl.org        - Automated bug and CVS messages
-   http://bioperl.org/MailList.shtml - About the mailing lists
+  bioperl-l@bioperl.org             - General discussion
+  http://bioperl.org/MailList.shtml - About the mailing lists
+
+
 
 =head2 Reporting Bugs
 
@@ -78,8 +80,10 @@ The rest of the documentation details each of the object methods. Internal metho
 
 =cut
 
+
 # Let's begin the code...
 
+
 package Bio::Index::EMBL;
 
 use vars qw($VERSION @ISA);
@@ -107,24 +111,6 @@ sub _version {
     return $VERSION;
 }
 
-=head2 _initialize
-
-  Title   : _initialize
-  Usage   : $index->_initialize
-  Function: Calls $index->SUPER::_initialize(), and then adds
-            the default id parser for EMBL files.
-  Example : 
-  Returns : 
-  Args    : 
-
-=cut
-
-sub _initialize {
-    my($self, $index_file, $write_flag) = @_;
-    
-    $self->SUPER::_initialize($index_file, $write_flag);
-}
-
 =head2 _index_file
 
   Title   : _index_file
@@ -200,6 +186,7 @@ sub _index_file {
  Returns : 
  Args    :
 
+
 =cut
 
 sub _file_format{
@@ -208,5 +195,16 @@ sub _file_format{
    return 'EMBL';
 }
 
+
+
 1;
 
+
+
+
+
+
+
+
+
+
