@@ -1,3 +1,4 @@
+# $Id: LocatableSeq.pm,v 1.5.2.1 2001/03/02 17:32:14 heikki Exp $
 #
 # BioPerl module for Bio::LocatableSeq
 #
@@ -15,6 +16,11 @@ Bio::LocatableSeq - A Sequence object with start/end points on it
 
 =head1 SYNOPSIS
 
+Give standard usage here
+
+=head1 DESCRIPTION
+
+
     # a normal sequence object
     $locseq->seq();
     $locseq->id();
@@ -22,28 +28,45 @@ Bio::LocatableSeq - A Sequence object with start/end points on it
     # has start,end points
     $locseq->start();
     $locseq->end();
-    
+
     # inheriets off RangeI, so range operations possible
 
     $locseq->overlaps($seqfeature);
 
-=head1 DESCRIPTION
+=head1 FEEDBACK
+
+
+=head2 Mailing Lists
+
+
+User feedback is an integral part of the evolution of this and other
+Bioperl modules. Send your comments and suggestions preferably to one
+of the Bioperl mailing lists.  Your participation is much appreciated.
+
+  bioperl-l@bioperl.org          - General discussion
+  http://bio.perl.org/MailList.html             - About the mailing lists
+
 
 The locatable sequence object was developed mainly because the 
 SimpleAlign object requires this functionality, and in the rewrite
 of the Sequence object we had to decide what to do with this.
 
 It is, to be honest, not well integrated with the rest of bioperl, for
-example, the ->trunc function does not return a LocatableSeq object,
+example, the trunc() function does not return a LocatableSeq object,
 as some might have thought. There are all sorts of nasty gotcha's about
 interactions between coordinate systems when these sort of objects are
 used. 
 
-Please post to the guts list for more about this, or contact Ewan Birney.
 
-=head1 CONTACT
+=head2 Reporting Bugs
+
+Report bugs to the Bioperl bug tracking system to help us keep track
+the bugs and their resolution.  Bug reports can be submitted via email
+or the web:
+
+  bioperl-bugs@bio.perl.org
+  http://bio.perl.org/bioperl-bugs/
 
-Ewan Birney <birney@ebi.ac.uk>
 
 =head1 APPENDIX
 
@@ -51,6 +74,7 @@ The rest of the documentation details each of the object methods. Internal metho
 
 =cut
 
+#'
 # Let the code begin...
 
 package Bio::LocatableSeq;
@@ -60,28 +84,26 @@ use strict;
 use Bio::Seq;
 use Bio::RangeI;
 
-# Object preamble - inheriets from Bio::Root::Object
-
-use Bio::Root::Object;
-
 @ISA = qw(Bio::Seq Bio::RangeI);
 
-# new() is inherited from Bio::Root::Object
+# new() is inherited from Bio::Root::RootI
 
 # _initialize is where the heavy stuff will happen when new is called
 
-sub _initialize {
-  my($self,@args) = @_;
+sub new {
+    my ($class, @args) = @_;
+    my $self = Bio::Seq->new(@args);
 
-  my ($start,$end,$strand) = $self->_rearrange( [qw(START END STRAND)],@args);
-  my $make = $self->SUPER::_initialize(@args);
- 
-  defined $start && $self->start($start);
-  defined $end   && $self->end($end);
-  defined $strand && $self->strand($strand);
+    bless $self, ref($class) || $class;
 
+    my ($start,$end,$strand) = $self->_rearrange( [qw(START END STRAND)],@args);
+    
+    defined $start && $self->start($start);
+    defined $end   && $self->end($end);
+    defined $strand && $self->strand($strand);
+    
 # set stuff in self from @args
- return $make; # success - we hope!
+    return $self; # success - we hope!
 }
 
 =head2 start
@@ -165,3 +187,4 @@ sub get_nse{
 
 }
 
+1;
