@@ -2,7 +2,7 @@
 # PACKAGE : Bio::Root::Object.pm
 # AUTHOR  : Steve A. Chervitz (sac@genome.stanford.edu)
 # CREATED : 23 July 1996
-# REVISION: $Id: Object.pm,v 1.8.2.2 2000/09/15 08:24:18 jgrg Exp $
+# REVISION: $Id: Object.pm,v 1.13 2001/01/30 19:22:06 lapp Exp $
 # STATUS  : Alpha
 #            
 # For documentation, run this module through pod2html 
@@ -36,12 +36,13 @@ use vars qw($ID $VERSION %Objects_created $Revision @ISA);
 
 @ISA = qw(Bio::Root::RootI);
 
+
 # %Objects_created can be used for tracking all objects created.
 # See _initialize() for details.
 
 $ID       = 'Bio::Root::Object';
 $VERSION  = 0.041;
-$Revision = '$Id: Object.pm,v 1.8.2.2 2000/09/15 08:24:18 jgrg Exp $';  #'
+$Revision = '$Id: Object.pm,v 1.13 2001/01/30 19:22:06 lapp Exp $';  #'
 
 ### POD Documentation:
 
@@ -51,6 +52,11 @@ Bio::Root::Object - A core Perl 5 object.
 
 =head1 SYNOPSIS
 
+B<Use of this module is deprecated. Wherever possible try to switch to
+Bio::Root::RootI as the root of your inheritance tree. Using this module as
+your root of inheritance will trigger a warning at object instantiation.
+This module is no longer being maintained and will eventually disappear.>
+
 =head2 Object Creation
 
     require Bio::Root::Object;
@@ -60,9 +66,11 @@ Bio::Root::Object - A core Perl 5 object.
 			         -parent  => $dad,
 			         -make    => 'full');
 
+
 See the L<new>() method for a complete description of parameters.
 See also L<USAGE>.
 
+
 =head1 INSTALLATION
 
 This module is included with the central Bioperl distribution:
@@ -198,12 +206,14 @@ Bio::Root::Err.pm objects can be reconstructed from the contents of C<$@>:
         printf "Stack trace: %s\n". $err->stack;
     }
 
+
 =head2 Demo Scripts
 
 Some demo script that illustrate working with Bio::Root::Objects are included with the distribution (see L<INSTALLATION>). These are also available at:
 
     http://bio.perl.org/Core/Examples/Root_object
 
+
 =head1 STRICTNESS & VERBOSITY
 
 There are two global variables that can be used to control sensitivity to
@@ -215,6 +225,7 @@ B<Bio::Root::Global.pm>.
 
   $VERBOSITY   - Regulates the amount of reporting by an object.
 
+
 The L<strict>() and L<verbose>() methods of B<Bio::Root::Object.pm>
 originally operated at the the object level, to permit individual
 strictness and verbosity levels for different objects. This level of
@@ -270,6 +281,7 @@ are sensitive to these values as indicated in the tables below:
 See the methods L<verbose>(), L<strict>(), L<throw>(), L<warn>(), and
 L<record_err>() for more details.
 
+
 =head1 DEPENDENCIES
 
 As the B<Bio::Root::Object.pm> does not inherit from any modules 
@@ -292,6 +304,7 @@ Since this module is at the root of potentially many different objects
 in a particular application, efficiency is important. Bio::Root::Object.pm is 
 intended to be a lightweight, lean and mean module. 
 
+
 =head1 FEEDBACK
 
 =head2 Mailing Lists 
@@ -301,7 +314,6 @@ Send your comments and suggestions preferably to one of the Bioperl mailing list
 Your participation is much appreciated.
 
   bioperl-l@bioperl.org             - General discussion
-  bioperl-guts-l@bioperl.org        - Automated bug and CVS messages
   http://bioperl.org/MailList.shtml - About the mailing lists
 
 =head2 Reporting Bugs
@@ -322,6 +334,7 @@ See the L<FEEDBACK> section for where to send bug reports and comments.
 
 Bio::Root::Object.pm, 0.041
 
+
 =head1 TODO
 
 =over 0
@@ -346,6 +359,7 @@ Consider incorporating a more widely-used Error/Exception module
   http://bio.perl.org/Projects/modules.html  - Online module documentation
   http://bio.perl.org/                       - Bioperl Project Homepage 
 
+
 =head2 Other Exception Modules
 
   Experimental::Exception.pm   - ftp://ftp.matematik.su.se/pub/teke/
@@ -371,6 +385,7 @@ modify it under the same terms as Perl itself.
 
 =cut
 
+
 # 
 ## 
 ### 
@@ -379,6 +394,7 @@ modify it under the same terms as Perl itself.
 ##
 # 
 
+
 =head1 APPENDIX
 
 Methods beginning with a leading underscore are considered private
@@ -388,10 +404,20 @@ for documentation purposes only.
 
 =cut
 
+#
+# This object is deprecated as the root of the inheritance tree, but some
+# modules depend on it as a legacy. We issue a deprecation warning for all
+# other modules.
+#
+my @inheriting_modules = ('Bio::Tools::Blast', 'Bio::Root::Object',
+			  'Bio::Root::IOManager');
+
+
 #######################################################
 #               CONSTRUCTOR/DESTRUCTOR                #
 #######################################################
 
+
 =head2 new
 
  Purpose   : Creates a blessed object reference (hash) for the indicated class
@@ -449,6 +475,7 @@ sub new {
     $self;
 }
 
+
 =head2 _initialize
 
  Purpose   : Initializes key Bio::Root::Object.pm data (name, parent, make, strict).
@@ -506,6 +533,11 @@ sub _initialize {
     local($^W) = 0;
     my($self, %param) = @_;
     
+    if(! grep { ref($self) =~ /$_/; } @inheriting_modules) {
+	$self->warn("Class " . ref($self) .
+		    " inherits from Bio::Root::Object, which is deprecated. ".
+		    "Try changing your inheritance to Bio::Root::RootI.");
+    }
     my($name, $parent, $make, $strict, $verbose, $obj, $record_err) = (
 	($param{-NAME}||$param{'-name'}), ($param{-PARENT}||$param{'-parent'}), 
 	($param{-MAKE}||$param{'-make'}), ($param{-STRICT}||$param{'-strict'}),
@@ -550,6 +582,8 @@ sub _initialize {
     return $make || 'default';
 }
 
+
+
 =head2 DESTROY
 
  Purpose   : Provides indication that the object is being reclaimed
@@ -576,6 +610,7 @@ sub DESTROY {
     $DEBUG==2 && print STDERR "DESTROY called in $ID for ${\$self->to_string} ($self)\n";
 }  
 
+
 =head2 destroy
 
  Purpose   : Clean up any resources allocated by the object and
@@ -634,6 +669,7 @@ sub destroy {
     $self->_remove_from_index if scalar %Objects_created;
 }
 
+
 =head2 _drop_child
 
  Usage     : $object->_drop_child(object_ref)
@@ -708,10 +744,13 @@ sub _drop_child {
     undef;
 }
 
+
 #################################################################
 #                    ACCESSORS & INSTANCE METHODS
 #################################################################
 
+
+
 =head2 name
 
  Usage     : $object->name([string]);
@@ -739,6 +778,7 @@ sub name {
     return defined $self->{'_name'} ? $self->{'_name'} : 'anonymous '.ref($self);
 }
 
+
 =head2 to_string
 
  Usage     : $object->to_string();
@@ -760,6 +800,7 @@ sub to_string {
     return sprintf "Object %s \"%s\"", ref($self), $self->name;
 }
 
+
 =head2 parent
 
  Usage     : $object->parent([object | 'null']);
@@ -801,6 +842,7 @@ sub parent {
     $self->{'_parent'};
 }
 
+
 =head2 src_obj
 
  Usage     : $object->src_obj([object | 'null']);
@@ -817,6 +859,7 @@ sub src_obj {
     $self->parent(@_);
 }
 
+
 =head2 has_name
 
  Usage     : $object->has_name();
@@ -839,6 +882,8 @@ See also   : L<name>()
 sub has_name { my $self = shift; return defined $self->{'_name'}; }
 #--------------
 
+
+
 =head2 make
 
  Usage     : $object->make([string]);
@@ -868,6 +913,7 @@ sub make {
     $self->{'_make'} || 'default'; 
 }
 
+
 =head2 err
 
  Usage     : $self->err([$data], [$delimit])
@@ -937,6 +983,7 @@ sub err {
     return $self->{'_err'}->get_all($data, $delimit );
 }	
 
+
 =head2 record_err
 
  Usage     : $object->record_err([0|1]);
@@ -966,6 +1013,7 @@ sub record_err {
     return $self->{'_record_err'} || 0;
 }
 
+
 =head2 err_state
 
  Usage     : $object->err_state();
@@ -986,6 +1034,7 @@ sub err_state {
     $self->{'_errState'} || 'OKAY'; 
 }
 
+
 =head2 clear_err
 
  Purpose   : To remove any error associated with the given object.
@@ -1002,6 +1051,10 @@ sub clear_err {
     $self->_set_err();
 }
 
+
+
+
+
 =head2 containment
 
  Usage     : $aref = $object->containment();
@@ -1042,6 +1095,7 @@ sub containment {
     return \@hierarchy;
 }
 
+
 =head2 set_stats
 
  Usage     : $object->set_stats(KEY => DATA [,KEY2 => DATA2])
@@ -1077,6 +1131,7 @@ sub set_stats {
     }
 }
 
+
 =head2 strict
 
  Usage     : $object->strict( [-2|-1|0|1|2] );
@@ -1130,6 +1185,9 @@ sub strict {
 	: (ref $self->{'_parent'} ? $self->{'_parent'}->strict : 0);
 }
 
+
+
+
 =head2 clone
 
  Purpose   : To deeply copy an object.
@@ -1171,6 +1229,8 @@ sub clone {
     $clone; 
 } 
 
+
+
 =head2 _set_clone
 
  Usage     : n/a; internal method used by _initialize()
@@ -1208,6 +1268,8 @@ sub _set_clone {
     $self->{'_err'}      = ref($obj->{'_err'}) and $obj->{'_err'}->clone;
 }
 
+
+
 =head2 verbose
 
  Usage     : $object->verbose([-1|0|1]);
@@ -1245,6 +1307,8 @@ sub verbose {
 	: (ref $self->{'_parent'} ? $self->{'_parent'}->verbose : 0);
 }
 
+
+
 =head1 I/O-RELATED METHODS (Delegated to B<Bio::Root::IOManager.pm>)
 
 =head2 _io
@@ -1260,6 +1324,8 @@ See also   : L<display>(), L<read>(), L<file>()
 sub _io  {  my $self = shift;   return $self->{'_io'}; }
 #-------
 
+
+
 =head2 _set_io
 
  Usage     : n/a; internal use only.
@@ -1281,6 +1347,8 @@ sub _set_io {
     $self->{'_io'} = new Bio::Root::IOManager(-PARENT=>$self);
 }
 
+
+
 =head2 set_display
 
  Usage     : $object->set_display( %named_parameters).
@@ -1315,6 +1383,7 @@ sub set_display {
    return $self->{'_io'}->fh;
 }
 
+
 =head2 display
 
  Usage     : $object->display( named parameters)
@@ -1378,6 +1447,9 @@ sub display {
     $self->{'_io'}->display(@param); 
 }
 
+
+
+
 =head2 _display_stats
 
  Usage     : n/a; called automatically by Bio::Root::Object::display(-SHOW=>'stats');
@@ -1409,6 +1481,8 @@ sub _display_stats {
     }
 }
 
+
+
 =head2 read
 
  Usage     : $object->read( named parameters)
@@ -1437,6 +1511,8 @@ sub read {
     $self->{'_io'}->read(@_); 
 }
 
+
+
 =head2 fh
 
  Usage     : $object->fh(['name'])
@@ -1458,6 +1534,7 @@ sub fh      {
     $self->{'_io'}->fh(@_); 
 }
 
+
 =head2 show
 
  Usage     : $object->show()
@@ -1480,6 +1557,8 @@ sub show    {
     $self->{'_io'}->show; 
 }
 
+
+
 =head2 file
 
  Usage     : $object->file()
@@ -1501,6 +1580,7 @@ sub file    {
     $self->{'_io'}->file(@_); 
 }
 
+
 =head2 compress_file
 
  Usage     : $object->compress_file([filename])
@@ -1523,6 +1603,8 @@ sub compress_file {
     $self->{'_io'}->compress_file(@_); 
 }
 
+
+
 =head2 uncompress_file
 
  Usage     : $object->uncompress_file([filename])
@@ -1544,6 +1626,7 @@ sub uncompress_file {
     $self->{'_io'}->uncompress_file(@_); 
 }
 
+
 =head2 delete_file
 
  Usage     : $object->delete_file([filename])
@@ -1566,6 +1649,7 @@ sub delete_file {
     $self->{'_io'}->delete_file(@_); 
 }
 
+
 =head2 file_date
 
  Usage     : $object->file_date( %named_parameters )
@@ -1589,8 +1673,11 @@ sub file_date {
     $self->{'_io'}->file_date(@_); 
 }
 
+
+
 =head1 EXPERIMENTAL METHODS 
 
+
 =head2 xref
 
  Usage     : $object->xref([object | 'null']);
@@ -1636,6 +1723,8 @@ sub xref  {
     $self->{'_xref'}; 
 }
 
+
+
 =head2 index
 
  Purpose   : To add an object to a package global hash of objects
@@ -1678,6 +1767,8 @@ sub _remove_from_index {
     undef $Objects_created{$class}->{$objName} if exists $Objects_created{$class}->{$objName};
 }
 
+
+
 =head2 find_object
 
  Purpose   : To obtain any object reference based on its unique name
@@ -1712,6 +1803,8 @@ sub find_object {
     $object;
 }
 
+
+
 =head2 has_warning
 
  Purpose   : Test whether or not an object has a non-fatal error (warning).
@@ -1734,6 +1827,8 @@ sub has_warning {
     0;
 }
 
+
+
 =head2 print_err
 
  Usage     : print_err([-WHERE=>FileHandle_object [,-SHOW=>msg|note|tech|stack] or any combo])
@@ -1767,6 +1862,8 @@ sub print_err {
 #    print "$ID: done print_err()\n";
 }
 
+
+
 =head2 err_string
 
  Usage     : err_string([-SHOW =>msg|note|tech|stack])
@@ -1803,6 +1900,9 @@ sub err_string {
     $out;
 }
 
+
+
+
 #################################################################
 #            DEPRECATED or HIGHLY EXPERIMENTAL METHODS
 #################################################################
@@ -1842,6 +1942,7 @@ sub terse {
     return $verbosity * -1;
 }
 
+
 #----------------------
 =head2 set_err_data()
 #----------------------
@@ -1893,6 +1994,8 @@ sub set_read {
     $self->{'_io'}->set_read(%param);
 }
 
+
+
 =head2 set_log_err
 
  Usage     : see Bio::Root::IOManager::set_log_err()
@@ -1916,9 +2019,11 @@ sub set_log_err {
     $self->{'_io'}->set_log_err(%param);
 }
 
+
 1;
 __END__
 
+
 #####################################################################################
 #                                  END OF CLASS                                     #
 #####################################################################################
@@ -1990,6 +2095,7 @@ all or some of the following fields:
 
 =cut
 
+
 MODIFICATION NOTES:
 -----------------------
 0.041, sac --- Thu Feb  4 03:50:58 1999
@@ -2037,3 +2143,4 @@ MODIFICATION NOTES:
   * Refined documentation in this and other Bio::Root modules
     (converted to use pod2html in Perl 5.004)
 
+
