@@ -1,3 +1,4 @@
+# $Id: CodonTable.pm,v 1.12.2.1 2001/03/02 22:48:03 heikki Exp $
 #
 # bioperl module for Bio::Tools::CodonTable
 #
@@ -11,7 +12,7 @@
 
 =head1 NAME
 
-  Bio::Tools::CodonTable - Bioperl codon table object
+Bio::Tools::CodonTable - Bioperl codon table object
 
 =head1 SYNOPSIS
 
@@ -20,7 +21,7 @@
   ambiguity codes for DNA, RNA and animo acids are recognized.
 
   # to use
-  use Bio::Tools:CodonTable;
+  use Bio::Tools::CodonTable;
 
   # defaults to ID 1 "Standard"
   $myCodonTable   = Bio::Tools::CodonTable->new();
@@ -30,7 +31,7 @@
   $myCodonTable->id(5);
 
   # examine codon table
-  print  join (' ', "The name of the codon table no.", $myCodonTable->id(4), 
+  print  join (' ', "The name of the codon table no.", $myCodonTable->id(4),
 	       "is:", $myCodonTable->name(), "\n");
 
   # translate a codon
@@ -38,7 +39,7 @@
   $aa = $myCodonTable->translate('act');
   $aa = $myCodonTable->translate('ytr');
 
-  # reverse translate an amino acid 
+  # reverse translate an amino acid
   @codons = $myCodonTable->revtranslate('A');
   @codons = $myCodonTable->revtranslate('Ser');
   @codons = $myCodonTable->revtranslate('Glx');
@@ -56,6 +57,7 @@ since that is what they try to represent. A bit more complete picture
 of the full complexity of codon usage in various taxonomic groups
 presented at the NCBI Genetic Codes Home page.
 
+
 CodonTable is a BioPerl class that knows all current translation
 tables that are used by primary nucleotide sequence databases
 (GenBank, EMBL and DDBJ). It provides methods to output information
@@ -68,12 +70,13 @@ conventions in EMBL and TREMBL databases.
 It is a nuisance to separate RNA and cDNA representations of nucleic
 acid transcripts. The CodonTable object accepts codons of both type as
 input and allows the user to set the mode for output when reverse
-translating. It default for output is DNA. 
+translating. Its default for output is DNA.
 
 Note: This class deals with individual codons and amino acids, only.
       Call it from your own objects to translate and reverse translate
       longer sequences.
 
+
 The amino acid codes are IUPAC recommendations for common amino acids:
 
           A           Ala            Alanine
@@ -98,7 +101,13 @@ The amino acid codes are IUPAC recommendations for common amino acids:
           V           Val            Valine
           B           Asx            Aspartic acid or Asparagine
           Z           Glx            Glutamine or Glutamic acid
-          X           Xaa            Any or unknown amino acid 
+          X           Xaa            Any or unknown amino acid
+
+
+It is worth noting that, "Bacterial" codon table no. 11 produces an
+polypeptide that is, confusingly, identical to the standard one. The
+only differences are in available initiator codons.
+
 
 NCBI Genetic Codes home page:
      http://www.ncbi.nlm.nih.gov/htbin-post/Taxonomy/wprintgc?mode=c
@@ -117,12 +126,11 @@ of these tables.
 =head2 Mailing Lists
 
 User feedback is an integral part of the evolution of this and other
-Bioperl modules. Send your comments and suggestions preferably to one
-of the Bioperl mailing lists.  Your participation is much appreciated.
+Bioperl modules. Send your comments and suggestions preferably to the
+Bioperl mailing lists  Your participation is much appreciated.
 
-   bioperl-l@bioperl.org             - General discussion
-   bioperl-guts-l@bioperl.org        - Automated bug and CVS messages
-   http://bioperl.org/MailList.shtml - About the mailing lists
+  bioperl-l@bioperl.org                         - General discussion
+  http://bio.perl.org/MailList.html             - About the mailing lists
 
 =head2 Reporting Bugs
 
@@ -136,11 +144,11 @@ report bugs to the Bioperl bug tracking system to help us keep track
 =head1 AUTHOR - Heikki Lehvaslaiho
 
 Email:  heikki@ebi.ac.uk
-Address: 
+Address:
 
      EMBL Outstation, European Bioinformatics Institute
      Wellcome Trust Genome Campus, Hinxton
-     Cambs. CB10 1SD, United Kingdom 
+     Cambs. CB10 1SD, United Kingdom
 
 =head1 APPENDIX
 
@@ -149,35 +157,30 @@ methods. Internal methods are usually preceded with a _
 
 =cut
 
+
 # Let the code begin...
 
 package Bio::Tools::CodonTable;
 use vars qw(@ISA);
 use strict;
 
-# Object preamble - inherits from Bio::Root::Object
-
-use Bio::Root::Object;
-
-@ISA = qw(Bio::Root::Object);
-
-# new() is inherited from Bio::Root::Object
-
-# _initialize is where the heavy stuff will happen when new is called
+# Object preamble - inherits from Bio::Root::RootI
+use Bio::Root::RootI;
+@ISA = qw(Bio::Root::RootI);
 
 {
 # first set internal values for all translation tables
 
 my @names =  #id
     (
-     'Standard', #1 
-     'Vertebrate Mitochondrial',#2 
-     'Yeast Mitochondrial',# 3 
+     'Standard', #1
+     'Vertebrate Mitochondrial',#2
+     'Yeast Mitochondrial',# 3
      'Mold, Protozoan, and CoelenterateMitochondrial and Mycoplasma/Spiroplasma',#4
-     'Invertebrate Mitochondrial',#5 
-     'Ciliate, Dasycladacean and Hexamita Nuclear',# 6 
+     'Invertebrate Mitochondrial',#5
+     'Ciliate, Dasycladacean and Hexamita Nuclear',# 6
        '', '',
-     'Echinoderm Mitochondrial',#9 
+     'Echinoderm Mitochondrial',#9
      'Euplotid Nuclear',#10
      '"Bacterial"',# 11
      'Alternative Yeast Nuclear',# 12
@@ -186,10 +189,12 @@ my @names =  #id
      'Blepharisma Nuclear',# 15
      'Chlorophycean Mitochondrial',# 16
        '', '',  '', '',
-     'Trematode Mitochondrial'# 21
+     'Trematode Mitochondrial',# 21
+     'Scenedesmus obliquus Mitochondrial', #22
+     'Thraustochytrium Mitochondrial' #23
      );
 
-my @tables = 
+my @tables =
     qw(
        FFLLSSSSYY**CC*WLLLLPPPPHHQQRRRRIIIMTTTTNNKKSSRRVVVVAAAADDEEGGGG
        FFLLSSSSYY**CCWWLLLLPPPPHHQQRRRRIIMMTTTTNNKKSS**VVVVAAAADDEEGGGG
@@ -208,9 +213,12 @@ my @tables =
        FFLLSSSSYY*LCC*WLLLLPPPPHHQQRRRRIIIMTTTTNNKKSSRRVVVVAAAADDEEGGGG
        '' '' '' ''
        FFLLSSSSYY**CCWWLLLLPPPPHHQQRRRRIIMMTTTTNNNKSSSSVVVVAAAADDEEGGGG   
+       FFLLSS*SYY*LCC*WLLLLPPPPHHQQRRRRIIIMTTTTNNKKSSRRVVVVAAAADDEEGGGG
+       FF*LSSSSYY**CC*WLLLLPPPPHHQQRRRRIIIMTTTTNNKKSSRRVVVVAAAADDEEGGGG
        );
 
-my @starts = 
+
+my @starts =
     qw(
        ---M---------------M---------------M----------------------------
        --------------------------------MMMM---------------M------------
@@ -229,8 +237,11 @@ my @starts =
        -----------------------------------M----------------------------
        '' ''  '' ''
        -----------------------------------M---------------M------------  
+       -----------------------------------M----------------------------
+       --------------------------------M--M---------------M------------
        );
 
+
 my @nucs = qw(t c a g);
 my ($x) = 0;
 my ($i, $j, $k);
@@ -242,26 +253,26 @@ for ($i=0;$i<4;$i++) {
 	    $codons->{$nucs[$i]. $nucs[$j]. $nucs[$k]} = $x;
 	    $trCol->{$x} = $nucs[$i]. $nucs[$j]. $nucs[$k];
 	    $x++;
-	}   
-    }   
+	}
+    }
 }
 
-my  %onecode =   
-    (Ala => 'A',     Asx => 'B',
-     Cys => 'C',     Asp => 'D',
-     Glu => 'E',     Phe => 'F',
-     Gly => 'G',     His => 'H',
-     Ile => 'I',     Lys => 'K',
-     Leu => 'L',     Met => 'M',
-     Asn => 'N',     Pro => 'P',
-     Gln => 'Q',     Arg => 'R',
-     Ser => 'S',     Thr => 'T',
-     Val => 'V',     Trp => 'W',
-     Xaa => 'X',     Tyr => 'Y',
-     Glx => 'Z',     Ter => '*'
+my  %onecode =
+    ('Ala' => 'A',     'Asx' => 'B',
+     'Cys' => 'C',     'Asp' => 'D',
+     'Glu' => 'E',     'Phe' => 'F',
+     'Gly' => 'G',     'His' => 'H',
+     'Ile' => 'I',     'Lys' => 'K',
+     'Leu' => 'L',     'Met' => 'M',
+     'Asn' => 'N',     'Pro' => 'P',
+     'Gln' => 'Q',     'Arg' => 'R',
+     'Ser' => 'S',     'Thr' => 'T',
+     'Val' => 'V',     'Trp' => 'W',
+     'Xaa' => 'X',     'Tyr' => 'Y',
+     'Glx' => 'Z',     'Ter' => '*'
      );
 
-my %iupac_dna = 
+my %iupac_dna =
     ( 'a' => [qw( a       )],
       'c' => [qw( c       )],
       'g' => [qw( g       )],
@@ -297,50 +308,49 @@ my %iupac_aa =
       '*' => ['*']
       );
 
-sub _initialize {
-    my($obj,@args) = @_;
-    
+sub new {
+    my($class,@args) = @_;
+    my $self = $class->SUPER::new(@args);
+
     my($id) =
-	$obj->_rearrange([qw(ID
+	$self->_rearrange([qw(ID
 			     )],
 			 @args);
-    
+
     $id = 1 if ( ! $id );
-    
-    my $make = $obj->SUPER::_initialize(@args);
-    
-    $id  && $obj->id($id);
-    
-    return $make; # success - we hope!
+    $id  && $self->id($id);
+    return $self; # success - we hope!
 }
 
+
 =head2 id
 
  Title   : id
  Usage   : $obj->id(3); $id_integer = $obj->id();
- Function: 
+ Function:
 
            Sets or returns the id of the translation table.  IDs are
            integers from 1 to 15, excluding 7 and 8 which have been
            removed as redundant. If an invalid ID is given the method
            returns 0, false.
 
- Example : 
- Returns : value of id, a scalar, 0 if not a valid 
+
+ Example :
+ Returns : value of id, a scalar, 0 if not a valid
  Args    : newvalue (optional)
 
 =cut
 
 sub id{
-   my ($obj,$value) = @_;
+   my ($self,$value) = @_;
    if( defined $value) {
        if (  !(defined $tables[$value-1]) or $tables[$value-1] eq '') {
-	   $obj->warn("Not a valid codon table ID [$value] ");
+	   $self->warn("Not a valid codon table ID [$value] ");
 	   $value = 0;
        }
-       $obj->{'id'} = $value;
+       $self->{'id'} = $value;
    }
-   return $obj->{'id'};
+   return $self->{'id'};
 }
 
 =head2 name
@@ -348,20 +358,22 @@ sub id{
  Title   : name
  Usage   : $obj->name()
  Function: returns the descriptive name of the translation table
- Example : 
+ Example :
  Returns : A string
  Args    : None
 
+
 =cut
 
 sub name{
-   my ($obj) = @_;
+   my ($self) = @_;
 
-   my ($id) = $obj->{'id'};
+   my ($id) = $self->{'id'};
    return $names[$id-1];
 
 }
 
+
 =head2 translate
 
  Title   : translate
@@ -381,18 +393,19 @@ sub name{
                acid (with exceptions above), return that, otherwise
                return empty string.
 
-           Return empty string for input strings that are not three
-           characters long.
+           Returns empty string for other input strings that are not
+           three characters long.
 
- Example : 
+ Example :
  Returns : One letter ambiguous IUPAC amino acid code
- Args    : a codon = a three character, ambiguous IUPAC nucleotide  string
+ Args    : a codon = a three character, ambiguous IUPAC nucleotide string
+
 
 =cut
 
 sub translate{
-   my ($obj, $value) = @_;
-   my ($id) = $obj->{'id'};
+   my ($self, $value) = @_;
+   my ($id) = $self->{'id'};
    my ($partial) = 0;
    my $result;
 
@@ -401,20 +414,20 @@ sub translate{
 
    if (length $value != 3 and length $value != 2) {
        return '';
-   } 
-   elsif ($value =~ /[^atgc]/i or length $value == 2 ) {  
+   }
+   elsif ($value =~ /[^atgc]/i or length $value == 2 ) {
        if (length $value == 2 ) {
 	   $value = $value. 'n';
 	   $partial = 1;
        }
        my @codons = _unambiquous_codons($value);
-       
-       my %aas =(); 
+
+       my %aas =();
        foreach my $codon (@codons) {
-	   $aas{substr($tables[$id-1],$codons->{$codon},1)} = 1;	   
+	   $aas{substr($tables[$id-1],$codons->{$codon},1)} = 1;	
        }
        #foreach my $x (keys %aas) {print "$x\n";} ###
-       
+
        my $count = scalar keys %aas;
        if ( $count == 1 ) {
 	   return (keys %aas)[0];
@@ -430,35 +443,36 @@ sub translate{
 	   }
        } else {
 	   $partial ? return '' :  return 'X';
-       } 
+       }
    } else {
-       return translate_strict (@_); 
+       return translate_strict (@_);
    }
 }
 
 =head2 translate_strict
 
- Title   : translate
+ Title   : translate_strict
  Usage   : $obj->translate_strict('ACT')
  Function: returns one letter amino acid code for a codon input
 
-           User is responsible to resolve ambiguous nucleotide codes
-           before calling this method. Returns 'X' for unknown codons
-           and an empty string for input strings that are not three
-           characters long.
+           Fast and simple translation. User is responsible to resolve
+           ambiguous nucleotide codes before calling this
+           method. Returns 'X' for unknown codons and an empty string
+           for input strings that are not three characters long.
 
            It is not recommended to use this method in a production
-           environment.
+           environment. Use method translate, instead.
 
- Example : 
+ Example :
  Returns : A string
  Args    : a codon = a three nucleotide character string
 
+
 =cut
 
 sub translate_strict{
-   my ($obj, $value) = @_;
-   my ($id) = $obj->{'id'};
+   my ($self, $value) = @_;
+   my ($id) = $self->{'id'};
 
    $value  = lc $value;
    $value  =~ tr/u/t/;
@@ -497,15 +511,15 @@ sub translate_strict{
 =cut
 
 sub revtranslate {
-    my ($obj, $value, $coding) = @_;
-    my ($id) = $obj->{'id'};
+    my ($self, $value, $coding) = @_;
+    my ($id) = $self->{'id'};
     my (@aas,  $p);
     my (@codons) = ();
 
     if (length($value) == 3 ) {
 	$value = lc $value;
 	$value = ucfirst $value;
-	$value = $onecode{$value}; 
+	$value = $onecode{$value};
     }
     if ( defined $value and $value =~ /[ARNDCQEGHILKMFPSTWYVBZX*]/ and length($value) == 1 ) {
 	$value = uc $value;
@@ -516,7 +530,7 @@ sub revtranslate {
 	    while ($tables[$id-1] =~ m/$aa/g) {
 		$p = pos $tables[$id-1];
 		push (@codons, $trCol->{--$p});
-	    } 
+	    }
 	}
     }
 
@@ -526,7 +540,7 @@ sub revtranslate {
 	}
     }
 
-    return @codons; 
+    return @codons;
 }
 
 =head2 is_start_codon
@@ -539,12 +553,13 @@ sub revtranslate {
  Returns : boolean
  Args    : codon
 
+
 =cut
 
 sub is_start_codon{
-   my ($obj, $value) = @_;
-   my ($id) = $obj->{'id'};
-   
+   my ($self, $value) = @_;
+   my ($id) = $self->{'id'};
+
    $value  = lc $value;
    $value  =~ tr/u/t/;
 
@@ -561,6 +576,8 @@ sub is_start_codon{
    }
 }
 
+
+
 =head2 is_ter_codon
 
  Title   : is_ter_codon
@@ -571,11 +588,12 @@ sub is_start_codon{
  Returns : boolean
  Args    : codon
 
+
 =cut
 
 sub is_ter_codon{
-   my ($obj, $value) = @_;
-   my ($id) = $obj->{'id'};
+   my ($self, $value) = @_;
+   my ($id) = $self->{'id'};
 
    $value  = lc $value;
    $value  =~ tr/u/t/;
@@ -603,11 +621,12 @@ sub is_ter_codon{
  Returns : boolean
  Args    : codon
 
+
 =cut
 
 sub is_unknown_codon{
-   my ($obj, $value) = @_;
-   my ($id) = $obj->{'id'};
+   my ($self, $value) = @_;
+   my ($id) = $self->{'id'};
 
    $value  = lc $value;
    $value  =~ tr/u/t/;
@@ -627,16 +646,16 @@ sub is_unknown_codon{
 
  Title   : _unambiquous_codons
  Usage   : @codons = _unambiquous_codons('ACN')
- Function: 
- Example : 
- Returns : array of strings (one letter unambiguous amino acid codes) 
+ Function:
+ Example :
+ Returns : array of strings (one letter unambiguous amino acid codes)
  Args    : a codon = a three IUPAC nucleotide character string
 
 =cut
 
 sub _unambiquous_codons{
     my ($value) = @_;
-    my @nts = ();   
+    my @nts = ();
     my @codons = ();
     my ($i, $j, $k);
     @nts = map { $iupac_dna{$_} }  split(//, $value) ;
@@ -651,7 +670,7 @@ sub _unambiquous_codons{
     return @codons;
 }
 
+
 }
 
 1;
-
