@@ -4,7 +4,7 @@
 #           and eventually for parsing, analyzing, running Fasta analyses.
 # AUTHOR  : Steve A. Chervitz (sac@genome.stanford.edu)
 # CREATED : 27 Mar 1998
-# REVISION: $Id: Fasta.pm,v 1.5.2.1 2000/09/15 08:24:23 jgrg Exp $
+# REVISION: $Id: Fasta.pm,v 1.5.4.1 2001/03/02 22:48:03 heikki Exp $
 # STATUS  : Alpha
 #
 # For documentation, run this module through pod2html 
@@ -38,6 +38,7 @@ use vars qw(@ISA @EXPORT @EXPORT_OK %EXPORT_TAGS
 $ID = 'Bio::Tools::Fasta';
 $VERSION  = 0.014; 
 
+
 ## Static FASTA object. 
 $Fasta = {};
 bless $Fasta, $ID;
@@ -45,6 +46,7 @@ $Fasta->{'_name'} = "Static Fasta object";
 
 $RawData = '';
 
+
 ## POD Documentation:
 
 =head1 NAME
@@ -103,6 +105,7 @@ Fasta sequence files as well as result files
 generated by FASTA sequence analysis, analogous to the way the
 B<Bio::Tools::Blast.pm> object is used for working with Blast output.
 
+
 =head2 References & Information about the FASTA program
 
 B<WEBSITES:>
@@ -110,6 +113,7 @@ B<WEBSITES:>
    ftp://ftp.virginia.edu/pub/fasta/    - FASTA software
    http://www2.ebi.ac.uk/fasta3/        - FASTA server at EBI
 
+
 B<PUBLICATIONS:> (with PubMed links)
 
   Pearson W.R. and Lipman, D.J. (1988). Improved tools for biological
@@ -134,6 +138,7 @@ and is also available from:
 Bio::Tools::Fasta.pm is a concrete class that inherits from B<Bio::Tools::SeqAnal.pm>.
 This module also relies on B<Bio::Seq.pm> for producing sequence objects.
 
+
 =head1 FEEDBACK
 
 =head2 Mailing Lists 
@@ -142,9 +147,9 @@ User feedback is an integral part of the evolution of this and other Bioperl mod
 Send your comments and suggestions preferably to one of the Bioperl mailing lists.
 Your participation is much appreciated.
 
-   bioperl-l@bioperl.org             - General discussion
-   bioperl-guts-l@bioperl.org        - Automated bug and CVS messages
-   http://bioperl.org/MailList.shtml - About the mailing lists
+    vsns-bcd-perl@lists.uni-bielefeld.de          - General discussion
+    vsns-bcd-perl-guts@lists.uni-bielefeld.de     - Technically-oriented discussion
+    http://bio.perl.org/MailList.html             - About the mailing lists
 
 =head2 Reporting Bugs
 
@@ -191,6 +196,7 @@ L<References & Information about the FASTA program>.
 
 =cut
 
+
 #
 ##
 ###
@@ -234,10 +240,13 @@ sub _initialize {
     $self->SUPER::_initialize( @param );
 }
 
+
+
 #####################################################################################
 ##                                  ACCESSORS                                      ##
 #####################################################################################
 
+
 =head2 parse
 
  Usage     : $fasta_obj->$parse( %named_parameters)
@@ -340,6 +349,7 @@ sub parse {
     $count;
 }
 
+
 =head2 _parse_seq_stream
 
  Usage     : n/a. Internal method called by parse()
@@ -389,6 +399,8 @@ sub _parse_seq_stream {
     
 }
 
+
+
 =head2 _get_parse_seq_func
 
  Usage     : n/a. Internal method called by _parse_seq_stream()
@@ -410,8 +422,8 @@ sub _parse_seq_stream {
            : If sequence objects are generated and an -EXEC_FUNC is supplied,
            : each object will be destroyed after calling this function.
            : This prevents memory usage problems for large runs.
-   
-See Also   : L<parse>(), L<_parse_seq_stream>(), B<Bio::Root::Object::_rearrange()>
+
+See Also   : L<parse>(), L<_parse_seq_stream>(), B<Bio::Root::Object::_rearrange>()
 
 =cut
 
@@ -549,6 +561,7 @@ sub _get_parse_seq_func {
     }
 }
 
+
 =head2 edit_id
 
  Usage     : $fasta_obj->edit_id()
@@ -570,6 +583,7 @@ sub edit_id {
     $self->{'_edit_id'};
 }
 
+
 =head2 edit_seqs
 
  Usage     : $fasta_obj->edit_seqs()
@@ -591,6 +605,8 @@ sub edit_seq {
     $self->{'_edit_seq'};
 }
 
+
+
 =head2 _set_id_desc
 
  Usage     : n/a. Internal method called by _get_parse_seq_func()
@@ -645,6 +661,7 @@ sub _set_id_desc {
     return ($id, join(' ',@desc).$desc_tag);
 }
 
+
 =head2 num_seqs
 
  Usage     : $fasta_obj->num_seqs()
@@ -659,6 +676,9 @@ sub _set_id_desc {
 sub num_seqs { my $self = shift;  $self->{'_seqCount'}; }
 #------------
 
+
+
+
 1;
 __END__
 
@@ -703,12 +723,14 @@ all or some of the following fields:
  More data members will be added when code for Fasta report
  processing is incorporated.
 
+
  INHERITED DATA MEMBERS 
 
 (See Bio::Tools::SeqAnal.pm for inherited data members.)
 
 =cut
 
+
 MODIFICATION NOTES:
 -------------------
 0.014, steve --- Wed Feb 17 02:22:26 1999
