@@ -1,3 +1,4 @@
+# $Id: Results.pm,v 1.10.2.1 2001/03/03 08:29:00 heikki Exp $
 #
 # Perl Module for HMMResults
 #
@@ -18,49 +19,52 @@ Bio::Tools::HMMER::Results - Object representing HMMER output results
    foreach $seq ( $res->each_Set ) {
        print "Sequence bit score is",$seq->bits,"\n";
        foreach $domain ( $seq->each_Domain ) {
-           print " Domain start ",$domain->start," end ",$domain->end," score ",$domain->bits,"\n";
+           print " Domain start ",$domain->start," end ",$domain->end,
+	   " score ",$domain->bits,"\n";
        }
    }
-  
+
    # new result object on a sequence/domain cutoff of 25 bits sequence, 15 bits domain
    $newresult = $res->filter_on_cutoff(25,15);
 
    # alternative way of getting out all domains directly
    foreach $domain ( $res->each_Domain ) {
-       print "Domain on ",$domain->seqname," with score ",$domain->bits," evalue ",$domain->evalue,"\n";
+       print "Domain on ",$domain->seqname," with score ",
+       $domain->bits," evalue ",$domain->evalue,"\n";
    }
 
 =head1 DESCRIPTION
 
-This object represents HMMER output, either from hmmsearch or hmmpfam. For hmmsearch,
-a series of HMMER::Set objects are made, one for each sequence, which have the 
-the bits score for the object. For hmmpfam searches, only one Set object is made.
+This object represents HMMER output, either from hmmsearch or
+hmmpfam. For hmmsearch, a series of HMMER::Set objects are made, one
+for each sequence, which have the the bits score for the object. For
+hmmpfam searches, only one Set object is made.
 
-These objects come from the original HMMResults modules used internally in Pfam,
-written by Ewan. Ewan then converted them to bioperl objects in 1999. That conversion
-is meant to be backwardly compatible, but may not be (caveat emptor).
+
+These objects come from the original HMMResults modules used
+internally in Pfam, written by Ewan. Ewan then converted them to
+bioperl objects in 1999. That conversion is meant to be backwardly
+compatible, but may not be (caveat emptor).
 
 =head1 FEEDBACK
 
 =head2 Mailing Lists
 
-User feedback is an integral part of the evolution of this
-and other Bioperl modules. Send your comments and suggestions preferably
- to one of the Bioperl mailing lists.
-Your participation is much appreciated.
+User feedback is an integral part of the evolution of this and other
+Bioperl modules. Send your comments and suggestions preferably to one
+of the Bioperl mailing lists.  Your participation is much appreciated.
 
-   bioperl-l@bioperl.org             - General discussion
-   bioperl-guts-l@bioperl.org        - Automated bug and CVS messages
-   http://bioperl.org/MailList.shtml - About the mailing lists
+  bioperl-l@bioperl.org                - General discussion
+  http://www.bioperl.org/MailList.html - About the mailing lists
 
 =head2 Reporting Bugs
 
 Report bugs to the Bioperl bug tracking system to help us keep track
- the bugs and their resolution.
- Bug reports can be submitted via email or the web:
+the bugs and their resolution.  Bug reports can be submitted via email
+or the web:
 
   bioperl-bugs@bio.perl.org
-  http://bio.perl.org/bioperl-bugs/
+  http://www.bioperl.org/bioperl-bugs/
 
 =head1 AUTHOR - Ewan Birney
 
@@ -80,55 +84,78 @@ use vars qw(@ISA);
 use Carp;
 use strict;
 
-use Bio::Root::Object;
+use Bio::Root::RootI;
+use Bio::Root::IO;
 use Bio::Tools::HMMER::Domain;
 use Bio::Tools::HMMER::Set;
-use FileHandle;
+use Bio::SeqAnalysisParserI;
+use Symbol;
 
-#
-# @ISA has our inheritance.
-#
+@ISA = qw(Bio::Root::RootI Bio::SeqAnalysisParserI);
 
-@ISA = ( 'Bio::Root::Object' );
+sub new {
+  my($class,@args) = @_;
 
-sub _initialize {
-  my($self,@args) = @_;
-
-  my $make = $self->SUPER::_initialize();
+  my $self = $class->SUPER::new(@args);
 
   $self->{'domain'} = []; # array of HMMUnits
   $self->{'seq'} = {};
 
-  my($file,$fh,$parsetype) = $self->_rearrange([qw(FILE
-						   FH
-						   TYPE
-						   )],
-					       @args);
-
-  if( !defined $file && ! defined $fh ) {
-      $self->throw("No file/filehandle definition to HMMER results");
-  }
-
-  if( defined $file ) {
-      $fh = new FileHandle;
-      $fh->open($file) || $self->throw("Could not open file [$file] $!");
-  }
+  my ($parsetype) = $self->_rearrange([qw(TYPE)],@args);
+  my $io = Bio::Root::IO->new(@args);
 
   if( !defined $parsetype ) {
       $self->throw("No parse type provided. should be hmmsearch or hmmpfam");
   }
 
   if( $parsetype eq 'hmmsearch' ) {
-      $self->_parse_hmmsearch($fh);
+      $self->_parse_hmmsearch($io->_fh());
   } elsif ( $parsetype eq 'hmmpfam' ) {
-      $self->_parse_hmmpfam($fh);
+      $self->_parse_hmmpfam($io->_fh());
   } else {
       $self->throw("Did not recoginise type $parsetype");
   } 
   
-  return $make; # success - we hope!
+  return $self; # success - we hope!
 }
 
+
+=head2 next_feature
+
+ Title   : next_feature
+ Usage   : while( my $feat = $res->next_feature ) { # do something }
+ Function: SeqAnalysisParserI implementing function
+ Example :
+ Returns : A Bio::SeqFeatureI compliant object, in this case, 
+           each DomainUnit object, ie, flattening the Sequence
+           aspect of this.
+ Args    : None
+
+
+=cut
+
+sub next_feature{
+   my ($self) = @_;
+
+   if( $self->{'_started_next_feature'} == 1 ) {
+       return shift @{$self->{'_next_feature_array'}};
+   } else {
+       $self->{'_started_next_feature'} = 1;
+       my @array;
+       foreach my $seq ( $self->each_Set() ) {
+	   foreach my $unit ( $seq->each_Domain() ) {
+	       push(@array,$unit);
+	   }
+       }
+       my $res = shift @array;
+       $self->{'_next_feature_array'} = \@array;
+       return $res;
+   }
+   
+   $self->throw("Should not reach here! Error!");
+}
+
+
 =head2 number
 
  Title   : number
@@ -155,6 +182,7 @@ sub number {
  Function: adds a domain to the results array. Mainly used internally.
  Args    : A Bio::Tools::HMMER::Domain
 
+
 =cut
 
 sub add_Domain {
@@ -172,6 +200,7 @@ sub add_Domain {
     push(@{$self->{'domain'}},$unit);
 }
 
+
 =head2 each_Domain
 
  Title   : each_Domain
@@ -180,6 +209,7 @@ sub add_Domain {
  Returns : array
  Args    : none
 
+
 =cut
 
 sub each_Domain {
@@ -193,6 +223,7 @@ sub each_Domain {
     return @arr;
 }
 
+
 =head2 domain_bits_cutoff_from_evalue
 
  Title   : domain_bits_cutoff_from_evalue
@@ -217,6 +248,7 @@ sub domain_bits_cutoff_from_evalue {
 
     @doms = $self->each_Domain;
 
+
     @doms = sort { $b->bits <=> $a->bits } @doms;
     $seen = 0;
     foreach $_ ( @doms ) {
@@ -246,6 +278,7 @@ sub domain_bits_cutoff_from_evalue {
     
 }
 
+
 sub dictate_hmm_acc {
     my $self = shift;
     my $acc = shift;
@@ -265,8 +298,10 @@ sub dictate_hmm_acc {
  Returns : 
  Args    :
 
+
 =cut
 
+
 sub write_FT_output {
     my $self = shift;
     my $file = shift;
@@ -296,8 +331,10 @@ sub write_FT_output {
  Args    : sequence cutoff and domain cutoff. in bits score
            if you want one cutoff, simply use same number both places
 
+
 =cut
 
+
 sub filter_on_cutoff {
     my $self = shift;
     my $seqthr = shift;
@@ -353,14 +390,16 @@ sub write_ascii_out {
 	$fh = \*STDOUT;
     }
 
+
     foreach $seq ( $self->each_Set()) {
 	foreach $unit ( $seq->each_Domain()) {
-	    print $fh sprintf("%s %4d %4d %s %4d %4d %4.2f %4.2g %s\n",$unit->seqname(),$unit->start(),$unit->end(),$unit->hmmacc,$unit->start_hmm,$unit->end_hmm,$unit->bits,$unit->evalue,$unit->hmmname);
+	    print $fh sprintf("%s %4d %4d %s %4d %4d %4.2f %4.2g %s\n",$unit->seqname(),$unit->start(),$unit->end(),$unit->hmmacc,$unit->hstart,$unit->hend,$unit->bits,$unit->evalue,$unit->hmmname);
 	}
     }
 	    
 }
 
+
 =head2 write_GDF_bits
 
  Title   : write_GDF_bits
@@ -370,7 +409,7 @@ sub write_ascii_out {
  Args    :
 
 =cut
-    
+
 sub write_GDF_bits {
     my $self = shift;
     my $seqt = shift;
@@ -468,6 +507,7 @@ sub write_GDF {
 	$file = \*STDOUT;
     }
 
+
     foreach $unit ( $self->eachHMMUnit() ) {
 	print $file sprintf("%-24s\t%6d\t%6d\t%15s\t%.1f\t%g\n",$unit->get_nse(),$unit->start(),$unit->end(),$unit->seqname(),$unit->bits(),$unit->evalue);
     }
@@ -495,10 +535,12 @@ sub highest_noise {
 	}
     }
 
+
     return ($noiseseq,$noisedom);
    
 }
 
+
 sub lowest_true {
     my $self = shift;
     my $seqt = shift;
@@ -530,10 +572,13 @@ sub lowest_true {
 	}
     }
 
+
     return ($trueseq,$truedom);
     
 }
 
+
+
 =head2 add_Set
 
  Title   : add_Set
@@ -542,6 +587,7 @@ sub lowest_true {
  Returns : 
  Args    :
 
+
 =cut
 
 sub add_Set {
@@ -557,6 +603,7 @@ sub add_Set {
     $self->{'seq'}->{$name} = $seq;
 }
 
+
 =head2 each_Set
 
  Title   : each_Set
@@ -565,18 +612,21 @@ sub add_Set {
  Returns : 
  Args    :
 
+
 =cut
 
 sub each_Set {
     my $self = shift;
     my (@array,$name);
 
+
     foreach $name ( keys %{$self->{'seq'}} ) {
 	push(@array,$self->{'seq'}->{$name});
     }
     return @array;
 }
 
+
 =head2 get_Set
 
  Title   : get_Set
@@ -585,6 +635,7 @@ sub each_Set {
  Returns : a HMMER::Set object
  Args    : name of the sequence
 
+
 =cut
 
 sub get_Set {
@@ -594,14 +645,16 @@ sub get_Set {
     return $self->{'seq'}->{$name};
 }
 
+
 =head2 _parse_hmmpfam
 
  Title   : _parse_hmmpfam
- Usage   : $res->parse_hmmpfam($filehandle)
+ Usage   : $res->_parse_hmmpfam($filehandle)
  Function:
  Returns : 
  Args    :
 
+
 =cut
 
 sub _parse_hmmpfam {
@@ -656,8 +709,8 @@ sub _parse_hmmpfam {
 		    $unit->hmmname  ($id);
 		    $unit->start    ($sqfrom);
 		    $unit->end      ($sqto);
-		    $unit->start_hmm($hmmf);
-		    $unit->end_hmm  ($hmmt);
+		    $unit->hstart($hmmf);
+		    $unit->hend  ($hmmt);
 		    $unit->bits     ($sc);
 		    $unit->evalue   ($ev);
 
@@ -679,6 +732,7 @@ sub _parse_hmmpfam {
 
 	    $_ = <$file>;
 
+
 	    # parses alignment lines. Icky as we have to break on the same line
 	    # that we need to read to place the alignment lines with the unit.
 
@@ -742,6 +796,7 @@ sub get_unit_nse {
     return undef;
 }
 
+
 =head2 _parse_hmmsearch
 
  Title   : _parse_hmmsearch
@@ -750,6 +805,7 @@ sub get_unit_nse {
  Returns : 
  Args    :
 
+
 =cut
 
 sub _parse_hmmsearch {
@@ -784,8 +840,8 @@ sub _parse_hmmsearch {
 	    $unit->start($sqfrom);
 	    $unit->end($sqto);
 	    $unit->bits($sc);
-	    $unit->start_hmm($hmmf);
-	    $unit->end_hmm($hmmt);
+	    $unit->hstart($hmmf);
+	    $unit->hend($hmmt);
 	    $unit->evalue($ev);
 	    $unit->seqbits($seqh{$id});
 	    $self->add_Domain($unit);
@@ -800,6 +856,10 @@ sub _parse_hmmsearch {
     return $count;
 }
 
+
+
+
 1;  # says use was ok
 __END__
 
+
