@@ -1,3 +1,4 @@
+# $Id: Set.pm,v 1.8.2.1 2001/03/03 08:29:00 heikki Exp $
 #
 # BioPerl module for Bio::Tools::HMMER::Set
 #
@@ -15,7 +16,6 @@ Bio::Tools::HMMER::Set - Set of identical domains from HMMER matches
 
 =head1 SYNOPSIS
 
-   
     # get a Set object probably from the results object
     print "Bits score over set ",$set->bits," evalue ",$set->evalue,"\n";
 
@@ -35,14 +35,12 @@ on the per domain scores, which you can get there).
 
 =head2 Mailing Lists
 
-User feedback is an integral part of the evolution of this
-and other Bioperl modules. Send your comments and suggestions preferably
- to one of the Bioperl mailing lists.
-Your participation is much appreciated.
+User feedback is an integral part of the evolution of this and other
+Bioperl modules. Send your comments and suggestions preferably to one
+of the Bioperl mailing lists.  Your participation is much appreciated.
 
-   bioperl-l@bioperl.org             - General discussion
-   bioperl-guts-l@bioperl.org        - Automated bug and CVS messages
-   http://bioperl.org/MailList.shtml - About the mailing lists
+  bioperl-l@bioperl.org                - General discussion
+  http://www.bioperl.org/MailList.html - About the mailing lists
 
 =head2 Reporting Bugs
 
@@ -51,8 +49,8 @@ the bugs and their resolution.
 
 Bug reports can be submitted via email or the web:
 
-  bioperl-bugs@bio.perl.org
-  http://bio.perl.org/bioperl-bugs/
+  bioperl-bugs@bioperl.org
+  http://www.bioperl.org/bioperl-bugs/
 
 =head1 AUTHOR - Ewan Birney
 
@@ -64,32 +62,24 @@ The rest of the documentation details each of the object methods. Internal metho
 
 =cut
 
+
 # Let the code begin...
 
+
 package Bio::Tools::HMMER::Set;
 use vars qw(@ISA);
 use strict;
 
-# Object preamble - inherits from Bio::Root::Object
-
-use Bio::Root::Object;
+use Bio::Root::RootI;
 use Bio::Tools::HMMER::Domain;
 
-#
-# Place functions/variables you want to *export*, ie be visible from the caller package into @EXPORT_OK
-#
+@ISA = qw( Bio::Root::RootI );
 
-#
-# @ISA has our inheritance.
-#
-
-@ISA = ( 'Bio::Root::Object' );
-
-sub _initialize {
-    my($self,@args) = @_;
-    my $make = $self->SUPER::_initialize(@args);
+sub new {
+    my($class,@args) = @_;
+    my $self = $class->SUPER::new(@args);    
     $self->{'domains'} = [];
-    return $make;
+    return $self;
 }
 
 =head2 add_Domain
@@ -105,6 +95,7 @@ sub _initialize {
 sub add_Domain{
    my ($self,$domain) = @_;
 
+
    if( ! defined $domain || ! $domain->isa("Bio::Tools::HMMER::Domain") ) {
        $self->throw("[$domain] is not a Bio::Tools::HMMER::Domain. aborting");
    }
@@ -121,6 +112,7 @@ sub add_Domain{
  Returns : array
  Args    : none
 
+
 =cut
 
 sub each_Domain{
@@ -138,6 +130,7 @@ sub each_Domain{
  Returns : value of name
  Args    : newvalue (optional)
 
+
 =cut
 
 sub name{
@@ -157,6 +150,7 @@ sub name{
  Returns : value of bits
  Args    : newvalue (optional)
 
+
 =cut
 
 sub bits{
@@ -178,6 +172,7 @@ sub bits{
  Returns : value of evalue
  Args    : newvalue (optional)
 
+
 =cut
 
 sub evalue{
@@ -189,6 +184,7 @@ sub evalue{
 
 }
 
+
 sub addHMMUnit {
     my $self = shift;
     my $unit = shift;
