@@ -1,3 +1,4 @@
+# $Id: IUPAC.pm,v 1.10.2.1 2001/03/03 08:28:58 heikki Exp $
 #
 # BioPerl module for IUPAC
 #
@@ -15,15 +16,15 @@ IUPAC - Generates unique Seq objects from an ambiguous Seq object
 
 =head1 SYNOPSIS
 
-use Bio::Seq;
-use Bio::Tools::IUPAC;
+ use Bio::Seq;
+ use Bio::Tools::IUPAC;
 
-my $ambiseq = new Bio::Seq (-seq => 'ARTCGUTGR', -type => 'Dna');
-my $stream  = new Bio::Tools::IUPAC($ambiseq);
+ my $ambiseq = new Bio::Seq (-seq => 'ARTCGUTGR', -type => 'Dna');
+ my $stream  = new Bio::Tools::IUPAC(-seq => $ambiseq);
 
-while ($uniqueseq = $stream->next_seq()) {
-    # process the unique Seq object.
-}
+ while ($uniqueseq = $stream->next_seq()) {
+     # process the unique Seq object.
+ }
 
 =head1 DESCRIPTION
 
@@ -88,6 +89,7 @@ the meaning shown below)
         Z        Glutamic Acid, Glutamine
         *        Terminator
 
+
         IUPAC-IUP AMINO ACID SYMBOLS:
           Biochem J. 1984 Apr 15; 219(2): 345-373
           Eur J Biochem. 1993 Apr 1; 213(1): 2
@@ -101,9 +103,8 @@ and other Bioperl modules. Send your comments and suggestions preferably
  to one of the Bioperl mailing lists.
 Your participation is much appreciated.
 
-   bioperl-l@bioperl.org             - General discussion
-   bioperl-guts-l@bioperl.org        - Automated bug and CVS messages
-   http://bioperl.org/MailList.shtml - About the mailing lists
+  bioperl-l@bioperl.org             - General discussion
+  http://www.bioperl.org/MailList.html - About the mailing lists
 
 =head2 Reporting Bugs
 
@@ -111,8 +112,8 @@ Report bugs to the Bioperl bug tracking system to help us keep track
  the bugs and their resolution.
  Bug reports can be submitted via email or the web:
 
-  bioperl-bugs@bio.perl.org
-  http://bio.perl.org/bioperl-bugs/
+  bioperl-bugs@bioperl.org
+  http://www.bioperl.org/bioperl-bugs/
 
 =head1 AUTHOR - Aaron Mackey
 
@@ -124,6 +125,7 @@ The rest of the documentation details each of the object methods. Internal metho
 
 =cut
 
+
 # Let the code begin...
 
 package Bio::Tools::IUPAC;
@@ -131,11 +133,8 @@ package Bio::Tools::IUPAC;
 use strict;
 use vars qw(@ISA $AUTOLOAD);
 
-# Object preamble - inherits from Bio::Root::Object
-
-use Bio::Root::Object;
-
-@ISA = qw(Bio::Root::Object);
+use Bio::Root::RootI;
+@ISA = qw(Bio::Root::RootI);
 
 =head2 new
 
@@ -146,20 +145,19 @@ use Bio::Root::Object;
            Seq objects on demand.
  Args    : an ambiguously coded Seq.pm object that has a specified 'type'
 
+
 =cut
 
-# new() is inherited from Bio::Root::Object
 
-# _initialize is where the heavy stuff will happen when new is called
+sub new {
+    my($class,@args) = @_;
+    my $self = $class->SUPER::new(@args);    
 
-sub _initialize {
-    my($self,@args) = @_;
-
-    my $make = $self->SUPER::_initialize;
-
-    # set stuff in self from @args
-
-    my $seq = shift @args;
+    my ($seq) = $self->_rearrange([qw(SEQ)],@args);
+    if((! defined($seq)) && @args && ref($args[0])) {
+	# parameter not passed as named parameter?
+	$seq = $args[0];
+    }
     $seq->isa('Bio::Seq') or $self->throw("Must supply a Seq.pm object to IUPAC!");
     $self->{'_SeqObj'} = $seq;
     if ($self->{'_SeqObj'}->moltype() =~ m/^[dr]na$/i ) { # nucleotide seq object
@@ -216,7 +214,7 @@ sub _initialize {
     $self->{'_string'} = [(0) x length($self->{'_SeqObj'}->seq())];
     scalar @{$self->{'_string'}} or $self->throw("Sequence has zero-length!");
     $self->{'_string'}->[0] = -1;
-    return $make; # success - we hope!
+    return $self;
 }
 
 =head2 next_seq
@@ -227,6 +225,7 @@ sub _initialize {
  Returns : a Seq.pm object
  Args    : none.
 
+
 =cut
 
 sub next_seq{
